@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2025	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -46,6 +46,7 @@ static int	forward_empty_query(POOL_CONNECTION * frontend, char *packet, int pac
 static int	forward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, int packetlen);
 static void process_clear_cache(POOL_CONNECTION_POOL * backend);
 static void clear_query_cache(void);
+static bool check_alter_role_statement(AlterRoleStmt *stmt);
 
 POOL_STATUS
 CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool command_complete)
@@ -111,8 +112,8 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 				if (session_context->query_context &&
 					session_context->query_context->parse_tree &&
 					is_start_transaction_query(session_context->query_context->parse_tree))
-					TSTATE(backend, i) = 'T';	/* we are inside a transaction */
 				{
+					TSTATE(backend, i) = 'T';	/* we are inside a transaction */
 					ereport(DEBUG1,
 							(errmsg("processing command complete"),
 							 errdetail("set transaction state to T")));
@@ -228,7 +229,19 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 			query = session_context->query_context->query_w_hex;
 			node = pool_get_parse_tree();
 			state = TSTATE(backend, MAIN_NODE_ID);
-			pool_handle_query_cache(backend, query, node, state);
+
+			/*
+			 * If some rows have been fetched by an execute with non 0 row option,
+			 * we do not create cache.
+			 */ 
+			pool_handle_query_cache(backend, query, node, state,
+									session_context->query_context->partial_fetch);
+
+			/*
+			 * CommandComplete guarantees that all rows have been fetched.  We
+			 * can unconditionally set atEnd flag.
+			 */
+			session_context->query_context->atEnd = true;
 		}
 	}
 
@@ -476,11 +489,44 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 		process_clear_cache(backend);
 	}
 	else if (IsA(node, AlterTableStmt) || IsA(node, AlterDatabaseStmt) ||
-			 IsA(node, AlterDatabaseSetStmt) || IsA(node, AlterRoleStmt))
+			 IsA(node, AlterDatabaseSetStmt))
 	{
 		/* Clear query cache */
 		process_clear_cache(backend);
 	}
+	else if (IsA(node, AlterRoleStmt))
+	{
+		if (check_alter_role_statement(castNode(AlterRoleStmt, node)))
+		{
+			/* Clear query cache */
+			process_clear_cache(backend);
+		}
+	}
+}
+
+/*
+ * Check whether the ALTER ROLE statement needs query cache invalidation.
+ * stmt must be AlterRoleStmt.
+ */
+static bool
+check_alter_role_statement(AlterRoleStmt *stmt)
+{
+	ListCell	*l;
+
+	foreach(l, stmt->options)
+	{
+		DefElem	*elm = (DefElem *) lfirst(l);
+
+		/*
+		 * We want to detect other than ALTER ROLE foo WITH PASSWORD or
+		 * WITH CONNECTION LIMIT case.  It does not change any privilege of the
+		 * role.
+		 */
+		if (strcmp(elm->defname, "password") &&
+			strcmp(elm->defname, "connectionlimit"))
+			return true;
+	}
+	return false;
 }
 
 /*
